require 'para_dice/string_die'
require 'para_dice/bag'
require 'para_dice/results/cancel_opposing'
require 'para_dice/results/split_faces'
require 'para_dice/results/hash'
require 'yaml'

# Another example of creating a custom set of dice
class GothamDice < ParaDice::Bag
  # path to yml file
  DATA_FILE_NAME  = 'data/gotham_dice.yml'

  # which faces are considered opposing, and thus cancel out in CancelOpposing
  #   readers
  OPPOSING        = [
      ['blank', 'blank'],
      ['pow', 'antipow'],
      ['clue', 'anticlue'],
      ['move', 'antimove'],
  ]
  # The Readers to use in order, in this case, use '_' to split face names
  #   (allowing single faces with multiple results), then cancel all of the
  #   opposing results, then present the results as a frequency hash
  DEFAULT_READERS = [
      ParaDice::Results::SplitFaces.new('_'),
      ParaDice::Results::CancelOpposing.new(OPPOSING),
      ParaDice::Results::Hash
  ]

  # @!attribute dice
  #   @return [Hash<String,Paradise::StringDie]
  attribute :dice, Hash[String => ParaDice::StringDie]

  # @!attribute default_readers
  #   @return [Array<#resolve>], defaults to DEFAULT_READERS
  attribute :default_readers, Array, default: ->(*a) { DEFAULT_READERS }

  # simple way to get a bag of dice with optional rng
  # @return [GothamDice]
  def self.get(rng = nil)
    hash = YAML.load_file(DATA_FILE_NAME)
    self.new(name: 'Gotham Dice', dice: hash, rng: rng)
  end
end
