require 'virtus'
require 'para_dice/die'
require 'para_dice/cup'
module ParaDice
  # Class to provide a convienent container for and roller of dice
  class Bag
    # Is a virtus.model
    include Virtus.model

    # @!attribute name
    #   @return [String] default: 'Dice Bag'
    attribute :name, String, default: 'Dice Bag'
    # @!attribute dice
    #   @return [Hash<String,Die>] default: {}
    attribute :dice, Hash[String => Die], default: {}
    # @!attribute default_readers
    #   @return [Array<#resolve>] default: []
    attribute :default_readers, Array, default: []
    # @!attribute rng
    #   @return [Random] default: Random.new
    attribute :rng, Random, default: ->(*a) { Random.new }

    # add a die to the bag
    # @param [Die]
    def add_die(die)
      dice[die.name] = die
    end

    # get an array of dice from an array of dice names
    # @param(*dice_names) array of dice names.
    def get_dice(*dice_names)
      dice_names.flatten.map { |die_name| dice[die_name.to_s] }
    end

    # given an array of dice_names, create a Cup using them and default_readers
    # @param(*dice_names) array of dice names.
    def get_cup(*dice_names)
      Cup.new(dice:    get_dice(*dice_names),
              rng:     rng,
              readers: default_readers
      )
    end
  end
end
