require 'para_dice/die'
module ParaDice
  # A Cup hold a selection of dice and a selection of readers.  It's main purpose
  #   is to provide a reusable component that you can call #roll on and get a
  #   complex result from
  class Cup
    # Includes Virtus.model
    include Virtus.model

    # @!attribute [rw] Dice
    #   @return [Array<ParaDice::Die]
    attribute :dice, Array[ParaDice::Die], default: []

    # @!attribute [rw] rng
    #   @return [Random]
    attribute :rng, Random, default: ->(*a) { Random.new }

    # @!attribute [rw] readers
    #   @return [Array<#resolvce>]
    attribute :readers, Array, default: []

    # roll reach die in cup and call each reader in turn on the results, return
    #   the result of the last reader
    # @param [Random,#rand] roll_rng default: self.rng
    def roll(roll_rng = rng)
      readers.reduce(roll!(roll_rng)) { |faces, reader| reader.resolve(faces) }
    end

    # roll each die in cup using roll_rng and return array of raw results from
    # @param [Random,#rand] roll_rng default: self.rng
    def roll!(roll_rng = rng)
      dice.map { |d| d.roll(roll_rng) }
    end
  end
end
