require 'para_dice/die'
module ParaDice
  # container module to put Face modules in
  module Faces
    # a module to be included into a ParaDice::Die or similar object.
    #   Uses an Array object to provide faces, and provides count and
    #   random_face methods
    # @note, these are strings, but the pattern could easily be symbols, numbers,
    #   json objects, etc.  the only difference would be in how you
    module Arrayed
      include ParaDice::Die

      # @!attribute faces
      #   @return Array<String>
      attribute :faces, Array[String], default: :quit_if_no_faces

      # returns size of array
      # @return [Fixnum]
      def face_count
        faces.size
      end

      # returns a random face from faces using rng
      # @return [String]
      def random_face(r = rng)
        faces.sample(random: r)
      end

      private
      def quit_if_no_faces
        raise ArgumentError.new('no :faces entry found in initialization hash')
      end
    end
  end
end
