require 'virtus'
module ParaDice
  # container module to put Face modules in
  module Faces
    # a module to be included into a ParaDice::Die or similar object.
    #   Uses a Range object to provide a faces, and provides special count and
    #   random_face options
    module Ranged
      include Virtus.module

      # @!attribute range
      #   return [Range]
      attribute :range, Range, default: :quit_if_no_range

      # @return [Fixnum] size of range
      def face_count
        range.size
      end

      # @note this can be an unexpectedly expensive action for large ranges,
      #   please use carefully
      # @return [Array<Fixnum>] result of range.to_a
      def faces
        @faces ||= range.to_a
      end

      # @param [nil,Random] random_face_rng defaults to self.rng
      # @return [Fixnum] random element of range
      def random_face(random_face_rng = rng)
        random_face_rng.rand(range.size) + range.first
      end

      private
      def quit_if_no_range
        raise ArgumentError.new('no :range entry found in initialization hash')
      end
    end
  end
end
