#  part of the paradice gem
module ParaDice
  #  operators that must have a #resolve method that can be passed an array
  module Results
    # Results class to implement a roll 5 and keep 3 style of play
    class AndKeep
      # default sort order.  passed to sort_by. defaults to a simple pass through
      DEFAULT_SORT_BY = ->(obj) { obj }

      # @!attribute [rw] :default_keep
      #   @return [Fixnum] the default number of dice to keep
      # @!attribute [rw] :default_low_results
      #   @return [Boolean] keep dice off of the low end of the order. false by default
      # @!attribute [rw] :default_sort_by
      #   @return [Proc] the sort by block defaults to passing the object through to sort
      attr_accessor :default_keep, :default_low_results, :default_sort_by
      # @param [Regex,String] default_keep
      def initialize(default_keep = nil)
        @default_keep = default_keep
      end

      # @param [Array<#to_s>] faces
      # @param [Regex,String] keep default: default_keep
      # @param [Boolean] low_results if true take the bottom of the order
      # @yield [obj] provides obj to call methods on to determine value for use
      #   in Array.order_by.  Numbers sort easy, names are trickier
      # @return [Array<String>]
      def resolve(faces, keep = default_keep, low_results = false, &blk)
        results_method = low_results ? :first : :last
        if block_given?
          faces.sort_by(&blk).to_a.send(results_method, keep)
        else
          faces.sort_by(&default_sort_by).to_a.send(results_method, keep)
        end
      end
    end
  end
end
