module ParaDice
  module Results
    # Number is meant to run a single operation across all faces of the results in
    # order to do things like add up all of the results.  This class gets inserted
    # as a result responder as opposed to the Module uses.  The default is to add all
    # of the elements together and return the result

    # @note this class could also be used for more complex operations using the
    #   reduce with operator and initial values.  Blocks seem like another likely
    #   refinement.  But I couldn't come with a reason to use them yet that didn't
    #   seem more confusing than coding a new result resolver
    class Number
      # If not overridden the initial value is 0
      DEFAULT_INITIAL  = 0
      # If not overridden the operator is :+
      DEFAULT_OPERATOR = :+
      # @!attribute [rw] default_initial_value
      #   @return [Numeric,Obj]
      # @!attribute [rw] default_operator
      #   @return [Symbol]
      attr_accessor :default_initial_value, :default_operator

      # @param [Numeric, Obj] default_operator
      # @param [Symbol]  default_operator
      def initialize(initial_val = DEFAULT_INITIAL, default_operator = DEFAULT_OPERATOR)
        @default_initial_value = initial_val
        @default_operator = default_operator
      end

      # @param [Array<Numeric,Object>] faces
      # @param [Numeric,Object] initial value, defaults to DEFAULT_INITIAL = 0
      # @param [Symbol] operator, defaults to DEFAULT_OPERATOR = :+
      # @return [Numeric, Object] result of inject with faces, initial_value, and operator
      def resolve(faces, initial_value = default_initial_value, operator = default_operator)
        faces.inject(initial_value, operator)
      end
    end
  end
end
