module ParaDice
  module Results
    # Results class to take faces and split them given a string or regexp
    class SplitFaces
      # @!attribute default_split_token
      #   @return [Regexp, String] used by split to separate token into pieces
      attr_accessor :default_split_token
      # when performing the split, use this regexp by default
      DEFAULT_SPLIT_TOKEN = /-|_/
      # @param [Regex,String] split_token defaults to DEFAULT_SPLIT_TOKEN=/-|_/
      def initialize(default_split_token = DEFAULT_SPLIT_TOKEN)
        @default_split_token = default_split_token
      end

      # @param [Array<#to_s>] faces
      # @param [Regex,String] split_token defaults to default_split_token
      # @return [Array<String>]
      def resolve(faces, split_token = default_split_token)
        faces.map { |f| f.to_s.split(split_token) }.flatten(1)
      end
    end
  end
end
