require 'para_dice/string_die'
require 'para_dice/results/cancel_opposing'
require 'para_dice/results/split_faces'
require 'para_dice/results/hash'
require 'para_dice/bag'
require 'yaml'

# A class for representing the game dice from Star Wars: Edge of the Empire,
#   Star Wars: Age of Rebellion, Star Wars: Force and Destiny.  Mostly here as
#   an example of how to create a custom dice bag.
# @note no claim is made for the name StarWarsDice
class StarWarsDice < ParaDice::Bag
  # yaml file containing dice definitions
  DATA_FILE_NAME  = 'data/star_wars_dice.yml'
  # opposite facing to be used by the reader CancelOpposing
  OPPOSING        = [
      ['blank', 'blank'],
      ['success', 'failure'],
      ['advantage', 'threat']
  ]
  # The normal reader order for StarWarsDice.  First split the face names using
  #   '_', then cancel opposing faces, finally return as a hash
  DEFAULT_READERS = [
      ParaDice::Results::SplitFaces.new('_'),
      ParaDice::Results::CancelOpposing.new(OPPOSING),
      ParaDice::Results::Hash
  ]

  # @!attribute [rw] dice
  #   @return [Hash<String,StringDie>]
  attribute :dice, Hash[String => ParaDice::StringDie]
  # @!attribute [rw] default_readers
  #   @return [Array<#resolve>]
  attribute :default_readers, Array, default: ->(*a) { DEFAULT_READERS }

  # load and return a bag of StarWarsDice
  def self.get
    hash = YAML.load_file(DATA_FILE_NAME)
    self.new(name: 'Star Wars Dice', dice: hash)
  end
end
